/**@file
 *
 *@brief	QRL Maker
 *@date		Sun,31 May,2015 - Tue,08 Sep,2015
 *@date		Fri,11 Sep,2015 - Tue,15 Sep,2015
 *@date		Mon,21 Sep,2015
 *@date		Sat,03 Oct,2015 - Mon,12 Oct,2015
 *@date		Wed,14 Oct,2015 - Sat,17 Oct,2015
 *@date		Thu,22 Oct,2015
 *@date		Mon,26 Oct,2015
 *@date		Sat,31 Oct,2015
 *@date		Sun,10 Jan,2016 - Mon,11 Jan,2016
 *@date		Thu,11 Feb,2016
 *@date		Sun,09 Jul,2017
 *@date		Sun,16 Jul,2017
 *@date		Sun,23 Jul,2017
 *@date		Wed,26 Jul,2017
 *@date		Thu,27 Jul,2017
 *@date		Sat,17 Mar,2018
 *@date		Fri,23 Nov,2018
 *@date		Mon,24 Dec,2018
 *@date		Fri,28 Dec,2018
 *@date		Sat,29 Dec,2018
 *@date		Sat,19 Jan,2019
 *@date		Fri,25 Jan,2019
 *@date		Sun,27 Jan,2019
 *@date		Tue,29 Jan,2019
 *@date		Sun,29 Nov,2020
 *@date		Sun,27 Dec,2020
 *@date		Fri,19 Feb,2021
 *@date		Sun,21 Feb,2021
 *@date		Tue,23 Feb,2021
 *@date		Mon,24 May,2021
 *@date		Fri,28 May,2021
 *@date		Sat,29 May,2021
 *@date		Thu,03 Jun,2021
 *@date		Sun,06 Jun,2021
 *@date		Sun,13 Jun,2021
 *@date		Wed,13 Jul,2022
 *@date		Fri,28 Apr,2023
 *@date		Sat,29 Apr,2023
 *@date		Sun,25 Jun,2023
 *
 *@author	Copyright(C)2015-2023 G-HAL. All rights reserved.
 *
 */
/**\mainpage
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.

 * Neither the name of the copyright holders nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include "settings.h"
#include "qrlmaker.h"
#include "mapeditor.h"
#include "form_mainform.h"
#include "qrl.h"
#include "qrlmap.h"
#include "mainwindow.h"
#include <QWidget>
#include <QListWidget>
#include <QString>
#include <QTreeWidget>
#if defined(DEBUG)
# include <QDateTime>
#endif

void Form_MainForm::setMainWindow(QRLMaker* const q, MainWindow* const mw) noexcept
{
	assert (NULL != q);
	assert (NULL != mw);

	qrlmaker = q;

	mapeditor = new(std::nothrow) MapEditor(qrlmaker, mw);
	assert (NULL != mapeditor);

	display_img = new(std::nothrow) ImageBuilder_QRLMap_DisplayImage(qrlmaker);
	assert (NULL != display_img);
	display_img->Set_scaling(1.0);

	save_img = new(std::nothrow) ImageBuilder_QRLMap_SaveImage(qrlmaker);
	assert (NULL != save_img);

	display_img->setMapeditor(mapeditor);
	save_img->setMapeditor(mapeditor);

	mainwindow = mw;

	return;
}

Form_MainForm::Form_MainForm(QWidget* parent) noexcept :
	QWidget(parent),
	qrlmaker(NULL),
	mapeditor(NULL),
	display_img(NULL),
	save_img(NULL),
  #if defined(DEBUG)
	interval_timer(NULL),
  #endif
	mouse_timer(NULL),
	mousePress_PendingEvent(NULL),
	mouseRelease_PendingEvent(NULL),
	mainwindow(NULL)
{
	selected_form_treeitem_name = "";
	selected_form_treeitem_name_tr = "";

	this->dragscroll.flag = false;

	this->cur_screen.scaling = 1.0;
	this->cur_screen.ofs.x = 0;
	this->cur_screen.ofs.y = 0;
	memcpy(&(this->old_screen), &(this->cur_screen), sizeof(this->old_screen));
	this->old_screen.scaling = 0.0;

	this->old_chipaddr.x = -1;
	this->old_chipaddr.y = -1;

	this->focus_addr.x = 0;
	this->focus_addr.y = 0;

	this->mode = Settings::EditorCommand::NORMAL;

	blink_color = -1;
  #if defined(DEBUG)
	blink_counter = 0;
	blink_counter_direction = +1;

	interval_timer_basetime = QDateTime::currentMSecsSinceEpoch();

	interval_timer = new(std::nothrow) QTimer;
	assert (NULL != interval_timer);
	interval_timer->setParent(this);
	interval_timer->setSingleShot(true);
	interval_timer->setInterval(250);
	connect(interval_timer, SIGNAL(timeout()), SLOT(update()));
	interval_timer->start(250);
  #endif

	mouse_waiting_timeout = false;
	mouse_press_count = 0;
	mouse_release_count = 0;
	mouse_timer = new(std::nothrow) QTimer;
	assert (NULL != mouse_timer);
	mouse_timer->setParent(this);
	mouse_timer->setSingleShot(true);
	connect(mouse_timer, &QTimer::timeout, this, &Form_MainForm::mouseTimeout);

	this->needRefresh = true;
	return;
}

Form_MainForm::~Form_MainForm(void) noexcept
{
	assert (NULL != qrlmaker);
	assert (NULL != mapeditor);
	assert (NULL != display_img);
	assert (NULL != save_img);
  #if defined(DEBUG)
	assert (NULL != interval_timer);
  #endif
	assert (NULL != mouse_timer);
	// assert (NULL != mousePress_PendingEvent); // Allocate only when using.
	// assert (NULL != mouseRelease_PendingEvent); // Allocate only when using.
	assert (NULL != mainwindow);

  #if defined(DEBUG)
	if (NULL != interval_timer) {
		interval_timer->stop();
	}
  #endif

	if (NULL != mouseRelease_PendingEvent) {
		delete mouseRelease_PendingEvent;
	}
	if (NULL != mousePress_PendingEvent) {
		delete mousePress_PendingEvent;
	}
	if (NULL != mouse_timer) {
		delete mouse_timer;
	}
  #if defined(DEBUG)
	if (NULL != interval_timer) {
		delete interval_timer;
	}
  #endif
	if (NULL != save_img) {
		delete save_img;
	}
	if (NULL != display_img) {
		delete display_img;
	}
	if (NULL != mapeditor) {
		delete mapeditor;
	}

	return;
}
// [ End of File ]
